// This file is part of the URDF PLUGIN for V-REP
//  
// Copyright 2006-2013 Dr. Marc Andreas Freese. All rights reserved. 
// marc@coppeliarobotics.com
// www.coppeliarobotics.com
// 
// A big thanks to Ignacio Tartavull for his precious help!
// 
// The URDF PLUGIN is licensed under the terms of GNU GPL:
// 
// -------------------------------------------------------------------
// The URDF PLUGIN is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// The URDF PLUGIN is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with the URDF PLUGIN.  If not, see <http://www.gnu.org/licenses/>.
// -------------------------------------------------------------------

//
// This file was automatically created for V-REP release V3.0.0 on January 9th 2013

#include "v_repExtUrdf.h"
#include "v_repLib.h"
#include <iostream>
#include <vector>
#include <bitset>
#include <sstream>
#include <boost/lexical_cast.hpp>
#include "7Vector.h"
#include "4X4Matrix.h"

#define _USE_MATH_DEFINES
#include <math.h>

#ifdef _WIN32
	#include <shlwapi.h>
	#pragma comment(lib, "Shlwapi.lib")
#endif /* _WIN32 */

#if defined (__linux) || defined (__APPLE__)
	#include <string.h>
	#define _stricmp(x,y) strcasecmp(x,y)
#endif

#define PLUGIN_VERSION 1

LIBRARY vrepLib; // the V-REP library that we will dynamically load and bind

// This is the plugin start routine (called just once, just after the plugin was loaded):
VREP_DLLEXPORT unsigned char v_repStart(void* reservedPointer,int reservedInt)
{
	// Dynamically load and bind V-REP functions:
	// ******************************************
	// 1. Figure out this plugin's directory:
	char curDirAndFile[1024];
#ifdef _WIN32
	GetModuleFileName(NULL,curDirAndFile,1023);
	PathRemoveFileSpec(curDirAndFile);
#elif defined (__linux) || defined (__APPLE__)
	getcwd(curDirAndFile, sizeof(curDirAndFile));
#endif
	std::string currentDirAndPath(curDirAndFile);
	// 2. Append the V-REP library's name:
	std::string temp(currentDirAndPath);
#ifdef _WIN32
	temp+="\\v_rep.dll";
#elif defined (__linux)
	temp+="/libv_rep.so";
#elif defined (__APPLE__)
	temp+="/libv_rep.dylib";
#endif /* __linux || __APPLE__ */
	// 3. Load the V-REP library:
	vrepLib=loadVrepLibrary(temp.c_str());
	if (vrepLib==NULL)
	{
		std::cout << "Error, could not find or correctly load the V-REP library. Cannot start 'Urdf' plugin.\n";
		return(0); // Means error, V-REP will unload this plugin
	}
	if (getVrepProcAddresses(vrepLib)==0)
	{
		std::cout << "Error, could not find all required functions in the V-REP library. Cannot start 'Urdf' plugin.\n";
		unloadVrepLibrary(vrepLib);
		return(0); // Means error, V-REP will unload this plugin
	}
	// ******************************************

	// Check the version of V-REP:
	// ******************************************
	int vrepVer;
	simGetIntegerParameter(sim_intparam_program_version,&vrepVer);
	if (vrepVer<20604) // if V-REP version is smaller than 2.06.04
	{
		std::cout << "Sorry, your V-REP copy is somewhat old. Cannot start 'Urdf' plugin.\n";
		unloadVrepLibrary(vrepLib);
		return(0); // Means error, V-REP will unload this plugin
	}
	// ******************************************

	simLockInterface(1);

	simAddModuleMenuEntry("",1,&menuItemHandle);
	simSetModuleMenuItemState(menuItemHandle,1,"URDF import...");

	simLockInterface(0);
	return(PLUGIN_VERSION); // initialization went fine, we return the version number of this plugin (can be queried with simGetModuleName)
}

// This is the plugin end routine (called just once, when V-REP is ending, i.e. releasing this plugin):
VREP_DLLEXPORT void v_repEnd()
{
	// Here you could handle various clean-up tasks

	unloadVrepLibrary(vrepLib); // release the library
}

// This is the plugin messaging routine (i.e. V-REP calls this function very often, with various messages):
VREP_DLLEXPORT void* v_repMessage(int message,int* auxiliaryData,void* customData,int* replyData)
{ // This is called quite often. Just watch out for messages/events you want to handle
	// Keep following 6 lines at the beginning and unchanged:
	simLockInterface(1);
	static bool refreshDlgFlag=true;
	int errorModeSaved;
	simGetIntegerParameter(sim_intparam_error_report_mode,&errorModeSaved);
	simSetIntegerParameter(sim_intparam_error_report_mode,sim_api_errormessage_ignore);
	void* retVal=NULL;

	// Here we can intercept many messages from V-REP (actually callbacks). Only the most important messages are listed here.
	// For a complete list of messages that you can intercept/react with, search for "sim_message_eventcallback"-type constants
	// in the V-REP user manual.

	if (message==sim_message_eventcallback_menuitemselected)
	{ // A custom menu bar entry was selected..
		if (auxiliaryData[0]==menuItemHandle)
		{ // we clicked our menu item
			simChar* pathAndFile=simFileDialog(sim_filedlg_type_load,"URDF import","N:\\drcsim-1.3\\models\\atlas\\","","URDF Files","urdf");
			if (pathAndFile!=NULL)
			{
				fileURDF=(std::string)pathAndFile;
				simReleaseBuffer(pathAndFile);
				simMsgBox(sim_msgbox_type_info,sim_msgbox_buttons_ok,"URDF import","The URDF importer plugin is an early beta version!!! Feel free to modify/improve the source code located in the 'programming' folder.");
				urdfOpen(fileURDF);
			}
		}
	}

	// Keep following unchanged:
	simSetIntegerParameter(sim_intparam_error_report_mode,errorModeSaved); // restore previous settings
	simLockInterface(0);
	return(retVal);
}

int urdfOpen(std::string filename)
{
	objectHandles.clear();
	objectNames.clear();
	tinyxml2::XMLDocument doc;
    
	if(doc.LoadFile((char*)filename.c_str())==tinyxml2::XML_NO_ERROR)
	{
		//file correctly read
		simAddStatusbarMessage("urdf file opened!!");
	}
	else
	{
		//something went wrong
		simAddStatusbarMessage("ERROR: file couldn't be opened");
		return 1;
	}
	tinyxml2::XMLElement* robotElement = doc.FirstChildElement();
	if(robotElement==0){return 1;}
    printf("%s\n",robotElement->Value());
	
	
	if((std::string)robotElement->Value()!="robot")
	{  
		//Error we have to ve working with a robot
		
	    simAddStatusbarMessage("There is no robot in this file \n");
		return 1;
	}
	
	printf("The robot name is %s \n",robotElement->Attribute("name"));//debug

	if(robotElement->NoChildren())
	{
		simAddStatusbarMessage("The robot has no elements");
		return 2;
	}

	//Analize links
	AnalizeLinks(robotElement);

	//Analize Joints
	AnalizeJoints(robotElement);
	
	
		
	return 0;
}


void AnalizeLinks(tinyxml2::XMLElement* robotElement)
{
	tinyxml2::XMLElement* linkElement = robotElement->FirstChildElement("link");
	while (linkElement)
	{
		std::string linkName = linkElement->Attribute("name");
		
		bool inertiaThere=false;
		bool collision=false;

		int handleOfVisualParent=-1;

		// INERTIA
		C4X4Matrix inertiaTr;
		inertiaTr.setIdentity();
		float mass=1.0f;
		float inertia[9]={1.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,1.0f};
		tinyxml2::XMLElement* inertialElement = linkElement->FirstChildElement("inertial");
		if(inertialElement!=NULL)
		{
			inertiaThere=true;
			tinyxml2::XMLElement* inertial_originElement = inertialElement->FirstChildElement("origin");
			if(inertial_originElement!=NULL)
			{
				std::string inertial_xyz("0 0 0");
				std::string inertial_rpy("0 0 0");
				if (inertial_originElement->Attribute("xyz") != NULL)
					inertial_xyz = inertial_originElement->Attribute("xyz");

				if (inertial_originElement->Attribute("rpy") != NULL)
					inertial_rpy = inertial_originElement->Attribute("rpy");

				inertiaTr.X.set(stringToArray(inertial_xyz));
				float* euler=stringToArray(inertial_rpy);
				inertiaTr.M.setEulerAngles(euler[0],euler[1],euler[2]);
			}

			tinyxml2::XMLElement* inertial_massElement = inertialElement->FirstChildElement("mass");
			if(inertial_massElement!=NULL)
			{
				if (inertial_massElement->Attribute("value") != NULL)
					mass = getFloat(inertial_massElement->Attribute("value"));
			}

			tinyxml2::XMLElement* inertial_inertiaElement = inertialElement->FirstChildElement("inertia");
			if(inertial_inertiaElement!=NULL)
			{
				if (inertial_inertiaElement->Attribute("ixx") != NULL)
					inertia[0] = getFloat(inertial_inertiaElement->Attribute("ixx"));
				if (inertial_inertiaElement->Attribute("ixy") != NULL)
					inertia[1] = getFloat(inertial_inertiaElement->Attribute("ixy"));
				if (inertial_inertiaElement->Attribute("ixz") != NULL)
					inertia[2] = getFloat(inertial_inertiaElement->Attribute("ixz"));
				if (inertial_inertiaElement->Attribute("iyy") != NULL)
					inertia[4] = getFloat(inertial_inertiaElement->Attribute("iyy"));
				if (inertial_inertiaElement->Attribute("iyz") != NULL)
					inertia[5] = getFloat(inertial_inertiaElement->Attribute("iyz"));
				if (inertial_inertiaElement->Attribute("izz") != NULL)
					inertia[8] = getFloat(inertial_inertiaElement->Attribute("izz"));
				inertia[3]=inertia[1];
				inertia[6]=inertia[2];
				inertia[7]=inertia[5];
			}
		}


		// COLLISION
		C4X4Matrix collisionTr;
		collisionTr.setIdentity();
		tinyxml2::XMLElement* collisionElement = linkElement->FirstChildElement("collision");
		if(collisionElement!=NULL)
		{
			collision=true;
			tinyxml2::XMLElement* collision_originElement = collisionElement->FirstChildElement("origin");
			if(collision_originElement!=NULL)
			{
				std::string collision_xyz("0 0 0");
				std::string collision_rpy("0 0 0");
				if (collision_originElement->Attribute("xyz") != NULL)
					collision_xyz = collision_originElement->Attribute("xyz");

				if (collision_originElement->Attribute("rpy") != NULL)
					collision_rpy = collision_originElement->Attribute("rpy");

				collisionTr.X.set(stringToArray(collision_xyz));
				float* euler=stringToArray(collision_rpy);
				collisionTr.M.setEulerAngles(euler[0],euler[1],euler[2]);
			}

			tinyxml2::XMLElement* collision_geometryElement = collisionElement->FirstChildElement("geometry");
			if(collision_geometryElement!=NULL)
			{
				tinyxml2::XMLElement* collision_geometry_meshElement = collision_geometryElement->FirstChildElement("mesh");
				if(collision_geometry_meshElement!=NULL)
				{
					std::string collision_geometry_meshElement_filename = collision_geometry_meshElement->Attribute("filename");
					//create mesh
					handleOfVisualParent=createMesh(collision_geometry_meshElement_filename,true,inertiaThere);
					C4X4Matrix m;
					simGetObjectPosition(handleOfVisualParent,-1,m.X.data);
					C3Vector euler;
					simGetObjectOrientation(handleOfVisualParent,-1,euler.data);
					m.M.setEulerAngles(euler);
					C4X4Matrix newConf=collisionTr*m;
					simSetObjectPosition(handleOfVisualParent,-1,newConf.X.data);
					simSetObjectOrientation(handleOfVisualParent,-1,newConf.M.getEulerAngles().data);
//					setObjectName(handleOfVisualParent,linkName.c_str());
				}
				tinyxml2::XMLElement* collision_geometry_boxElement = collision_geometryElement->FirstChildElement("box");
				if(collision_geometry_boxElement!=NULL)
				{
					std::string size("0.1 0.1 0.1");
					if (collision_geometry_boxElement->Attribute("size")!=NULL)
						size = collision_geometry_boxElement->Attribute("size");
					handleOfVisualParent=createPureShape(size,0,true,inertiaThere);
					simSetObjectPosition(handleOfVisualParent,-1,collisionTr.X.data);
					simSetObjectOrientation(handleOfVisualParent,-1,collisionTr.M.getEulerAngles().data);
//					setObjectName(handleOfVisualParent,linkName.c_str());
				} 
				tinyxml2::XMLElement* collision_geometry_sphereElement = collision_geometryElement->FirstChildElement("sphere");
				if(collision_geometry_sphereElement!=NULL)
				{
					std::string radius("0.1");
					if (collision_geometry_sphereElement->Attribute("radius")!=NULL)
						radius = collision_geometry_sphereElement->Attribute("radius");
					handleOfVisualParent=createPureShape(radius+" "+radius+" "+radius,1,true,inertiaThere);
					simSetObjectPosition(handleOfVisualParent,-1,collisionTr.X.data);
					simSetObjectOrientation(handleOfVisualParent,-1,collisionTr.M.getEulerAngles().data);
//					setObjectName(handleOfVisualParent,linkName.c_str());
				} 
				tinyxml2::XMLElement* collision_geometry_cylinderElement = collision_geometryElement->FirstChildElement("cylinder");
				if(collision_geometry_cylinderElement!=NULL)
				{
					std::string radius("0.1");
					std::string length("0.1");
					if (collision_geometry_cylinderElement->Attribute("radius")!=NULL)
						radius = collision_geometry_cylinderElement->Attribute("radius");
					if (collision_geometry_cylinderElement->Attribute("length")!=NULL)
						length = collision_geometry_cylinderElement->Attribute("length");
					handleOfVisualParent=createPureShape(radius+" "+radius+" "+length,2,true,inertiaThere);
					simSetObjectPosition(handleOfVisualParent,-1,collisionTr.X.data);
					simSetObjectOrientation(handleOfVisualParent,-1,collisionTr.M.getEulerAngles().data);
//					setObjectName(handleOfVisualParent,linkName.c_str());
				} 
			}
		}

		if (inertiaThere)
		{ // now apply the inertia. If we have no collision shape, we create a dummy collision shape that doesn't collide!
			if (handleOfVisualParent==-1)
			{
				handleOfVisualParent=createPureShape("0.05 0.05 0.05",0,false,inertiaThere);
//				setObjectName(handleOfVisualParent,linkName.c_str());
			}
			C4X4Matrix m;
			simGetObjectPosition(handleOfVisualParent,-1,m.X.data);
			C3Vector euler;
			simGetObjectOrientation(handleOfVisualParent,-1,euler.data);
			m.M.setEulerAngles(euler);
			C4X4Matrix x(m.getInverse()*inertiaTr);
			C3Vector com;
			com.clear();
			float i[12]={x.M(0,0),x.M(0,1),x.M(0,2),x.X(0),x.M(1,0),x.M(1,1),x.M(1,2),x.X(1),x.M(2,0),x.M(2,1),x.M(2,2),x.X(2)};
			simSetShapeMassAndInertia(handleOfVisualParent,mass,inertia,com.data,i);
		}

	    //VISUAL
		C4X4Matrix visualTr;
		visualTr.setIdentity();
		tinyxml2::XMLElement* visualElement = linkElement->FirstChildElement("visual");
		if(visualElement != NULL)
		{
			tinyxml2::XMLElement* visual_originElement = visualElement->FirstChildElement("origin");
			if(visual_originElement != NULL)
			{
				std::string visual_xyz("0 0 0");
				std::string visual_rpy("0 0 0");
				if (visual_originElement->Attribute("xyz") != NULL)
					visual_xyz = visual_originElement->Attribute("xyz");

				if (visual_originElement->Attribute("rpy") != NULL)
					visual_rpy = visual_originElement->Attribute("rpy");

				visualTr.X.set(stringToArray(visual_xyz));
				float* euler=stringToArray(visual_rpy);
				visualTr.M.setEulerAngles(euler[0],euler[1],euler[2]);
			}

			tinyxml2::XMLElement* visual_geometryElement = visualElement->FirstChildElement("geometry");
			if(visual_geometryElement!=NULL)
			{
				int geometryHandle=-1;
				tinyxml2::XMLElement* visual_geometry_meshElement = visual_geometryElement->FirstChildElement("mesh");
				if(visual_geometry_meshElement!=NULL)
				{
					std::string visual_geometry_meshElement_filename = visual_geometry_meshElement->Attribute("filename");
					//create mesh
					geometryHandle=createMesh(visual_geometry_meshElement_filename,false,false);
					C4X4Matrix m;
					simGetObjectPosition(geometryHandle,-1,m.X.data);
					C3Vector euler;
					simGetObjectOrientation(geometryHandle,-1,euler.data);
					m.M.setEulerAngles(euler);
					C4X4Matrix newConf=visualTr*m;
					simSetObjectPosition(geometryHandle,-1,newConf.X.data);
					simSetObjectOrientation(geometryHandle,-1,newConf.M.getEulerAngles().data);
				}
				tinyxml2::XMLElement* visual_geometry_boxElement = visual_geometryElement->FirstChildElement("box");
				if(visual_geometry_boxElement!=NULL)
				{
					std::string size("0.1 0.1 0.1");
					if (visual_geometry_boxElement->Attribute("size")!=NULL)
						size = visual_geometry_boxElement->Attribute("size");
					geometryHandle=createPureShape(size,0,false,false);
					simSetObjectPosition(geometryHandle,-1,visualTr.X.data);
					simSetObjectOrientation(geometryHandle,-1,visualTr.M.getEulerAngles().data);
				} 
				tinyxml2::XMLElement* visual_geometry_sphereElement = visual_geometryElement->FirstChildElement("sphere");
				if(visual_geometry_sphereElement!=NULL)
				{
					std::string radius("0.1");
					if (visual_geometry_sphereElement->Attribute("radius")!=NULL)
						radius = visual_geometry_sphereElement->Attribute("radius");
					geometryHandle=createPureShape(radius+" "+radius+" "+radius,1,false,false);
					simSetObjectPosition(geometryHandle,-1,visualTr.X.data);
					simSetObjectOrientation(geometryHandle,-1,visualTr.M.getEulerAngles().data);
				} 
				tinyxml2::XMLElement* visual_geometry_cylinderElement = visual_geometryElement->FirstChildElement("cylinder");
				if(visual_geometry_cylinderElement!=NULL)
				{
					std::string radius("0.1");
					std::string length("0.1");
					if (visual_geometry_cylinderElement->Attribute("radius")!=NULL)
						radius = visual_geometry_cylinderElement->Attribute("radius");
					if (visual_geometry_cylinderElement->Attribute("length")!=NULL)
						length = visual_geometry_cylinderElement->Attribute("length");
					geometryHandle=createPureShape(radius+" "+radius+" "+length,2,false,false);
					simSetObjectPosition(geometryHandle,-1,visualTr.X.data);
					simSetObjectOrientation(geometryHandle,-1,visualTr.M.getEulerAngles().data);
				} 

				// If we have just a visual, then it becomes the main object of the link and gets its name.
				// If however we have a collision object, or an inertia, then the visual simply gets attached to it and has no specific name

				if (handleOfVisualParent==-1)
					handleOfVisualParent=geometryHandle;
				else
				{ // now we correctly attach the visual shape to the collision/inertia object:
/*
					C4X4Matrix parentTr;
					simGetObjectPosition(handleOfVisualParent,-1,parentTr.X.data);
					C3Vector euler;
					simGetObjectOrientation(handleOfVisualParent,-1,euler.data);
					parentTr.M.setEulerAngles(euler);

					C4X4Matrix visTr;
					simGetObjectPosition(geometryHandle,-1,visTr.X.data);
					C3Vector euler;
					simGetObjectOrientation(geometryHandle,-1,euler.data);
					visTr.M.setEulerAngles(euler);
					C4X4Matrix x(parentTr.getInverse()*visTr);
*/
					simSetObjectParent(geometryHandle,handleOfVisualParent,1);
				}
			}
		}

		if (handleOfVisualParent==-1)
		{ // this happens with "world" for instance: an empty link
			handleOfVisualParent=simCreateDummy(0.02f,NULL);
		}

		setObjectName(handleOfVisualParent,linkName.c_str());


		
		linkElement = linkElement->NextSiblingElement("link"); //moves to the next link
	}
}

void AnalizeJoints(tinyxml2::XMLElement* robotElement)
{
	tinyxml2::XMLElement* jointElement = robotElement->FirstChildElement("joint");
	while (jointElement)
	{
		std::string jointType = jointElement->Attribute("type");
		std::string jointName = jointElement->Attribute("name");
		
		C4X4Matrix originTr;
		originTr.setIdentity();
		tinyxml2::XMLElement* joint_originElement = jointElement->FirstChildElement("origin");
		if(joint_originElement!=NULL)
		{
			std::string origin_xyz("0 0 0");
			std::string origin_rpy("0 0 0");
			if (joint_originElement->Attribute("xyz") != NULL)
				origin_xyz = joint_originElement->Attribute("xyz");

			if (joint_originElement->Attribute("rpy") != NULL)
				origin_rpy = joint_originElement->Attribute("rpy");

			originTr.X.set(stringToArray(origin_xyz));
			float* euler=stringToArray(origin_rpy);
			originTr.M.setEulerAngles(euler[0],euler[1],euler[2]);
		}

		C3Vector axis(1.0f,0.0f,0.0f);
		tinyxml2::XMLElement* joint_axisElement = jointElement->FirstChildElement("axis");
		if(joint_axisElement!=NULL)
		{
			std::string axis_xyz("1 0 0");
			if (joint_axisElement->Attribute("xyz") != NULL)
				axis_xyz = joint_axisElement->Attribute("xyz");
			axis.set(stringToArray(axis_xyz));
		}
				

		std::string parent;
		tinyxml2::XMLElement* joint_parentElement = jointElement->FirstChildElement("parent");
		if(joint_parentElement!=NULL)
		{
			parent = joint_parentElement->Attribute("link");
		}
		std::string child;
		tinyxml2::XMLElement* joint_childElement = jointElement->FirstChildElement("child");
		if(joint_childElement!=NULL)
		{
			child = joint_childElement->Attribute("link");
		}

		float limits[2]={0.0f,0.0f};
		tinyxml2::XMLElement* joint_limitElement = jointElement->FirstChildElement("limit");
		if(joint_limitElement!=NULL)
		{
			if (joint_limitElement->Attribute("lower")!=NULL)
				limits[0]=getFloat(joint_limitElement->Attribute("lower"));
			if (joint_limitElement->Attribute("upper")!=NULL)
				limits[1]=getFloat(joint_limitElement->Attribute("upper"));
		}

		int parentLink=getHandleFromName(parent.c_str());
		int childLink=getHandleFromName(child.c_str());
		int parentParentHandle=simGetObjectParent(parentLink);
/*
		simInt nParent;
		if(parent == "world" ){ nParent = -1;}
		else
		{
			nParent = simGetObjectHandle(parent.c_str());
		}
			simInt nChild = simGetObjectHandle(child.c_str());
	*/	

		//Debug simAddStatusbarMessage((parent+ " is the parent of " + child).c_str());

		////Create the joint
		int nJointType=-1;
		bool cont=false;
		if (jointType == "revolute")
			nJointType = sim_joint_revolute_subtype;
		if (jointType == "continuous")
		{
			nJointType = sim_joint_revolute_subtype;
			cont=true;
		}
		if (jointType == "prismatic")
			nJointType = sim_joint_prismatic_subtype;

		int objectHandle;
		if (nJointType!=-1)
		{
			float interval[2]={limits[0],limits[1]-limits[0]};
//			objectHandle=simCreateJoint(nJointType,sim_jointmode_force,0,NULL,NULL,NULL);
			objectHandle=simCreateJoint(nJointType,sim_jointmode_passive,0,NULL,NULL,NULL); // for the time being, we have the joints passive
			if (!cont)
				simSetJointInterval(objectHandle,0,interval);
		}
		else
			objectHandle=simCreateDummy(0.02f,NULL);

		C4X4Matrix x(originTr);
		
		if (nJointType!=-1)
		{
			// The joint is at pos/orientation originTr. But there is an additional rotational transformation for the axis. Calculate the joint configuration:
			C4X4Matrix jointAxisMatrix;
			jointAxisMatrix.setIdentity();
			C3Vector rotAxis;
			float rotAngle=0.0f;
			if (axis(2)<1.0f)
			{
				if (axis(2)<=-1.0f)
					rotAngle=3.14159265359f;
				else
					rotAngle=acosf(axis(2));
				rotAxis(0)=-axis(1);
				rotAxis(1)=axis(0);
				rotAxis(2)=0.0f;
				rotAxis.normalize();
				C7Vector m(jointAxisMatrix);
				float alpha=-atan2(rotAxis(1),rotAxis(0));
				float beta=atan2(-sqrt(rotAxis(0)*rotAxis(0)+rotAxis(1)*rotAxis(1)),rotAxis(2));
				C7Vector r;
				r.X.clear();
				r.Q.setEulerAngles(0.0f,0.0f,alpha);
				m=r*m;
				r.Q.setEulerAngles(0.0f,beta,0.0f);
				m=r*m;
				r.Q.setEulerAngles(0.0f,0.0f,rotAngle);
				m=r*m;
				r.Q.setEulerAngles(0.0f,-beta,0.0f);
				m=r*m;
				r.Q.setEulerAngles(0.0f,0.0f,-alpha);
				m=r*m;
				jointAxisMatrix=m.getMatrix();
	//			simRotateAroundAxis(jointAxisMatrix,rotAxis,axPos,rotAngle,jointAxisMatrix);
			}
			x=originTr*jointAxisMatrix;
		}

		simSetObjectPosition(objectHandle,-1,x.X.data);
		simSetObjectOrientation(objectHandle,-1,x.M.getEulerAngles().data);


		setObjectName(objectHandle,jointName.c_str());
		// The trick in following 2 lines only works if we have a consistent tree description from base to leaves: (change this later!)
		simSetObjectParent(objectHandle,parentParentHandle,false);
		simSetObjectParent(objectHandle,parentLink,true);
		simSetObjectParent(childLink,objectHandle,false);


		jointElement = jointElement->NextSiblingElement("joint"); //moves to the next joint
	}
}

int createMesh(std::string filename,bool collision,bool dynamic,std::string packagePath)
{
	//->manipulation of the filename
	//filename is in the form: package://multisense_sl/meshes/base_link.STL   or package://atlas/meshes/l_clav.dae" 
	//the global variable fileURDF is in the form: N:/drcsim-1.3/models/atlas/model.urdf
		int cutPackagePath = packagePath.find_last_of("/");

		////debug
		//char buffer [33];
		//itoa(cutPackagePath,buffer,10);
		//simAddStatusbarMessage(buffer);
		////debug

		packagePath=packagePath.substr(0,cutPackagePath);
		cutPackagePath = packagePath.find_last_of("/");
		packagePath=packagePath.substr(0,cutPackagePath);  //I do it twice to get  N:/drcsim-1.3/models/ so I can replace it for package://

		filename=filename.substr(9,filename.size()); //to delete de package:/ part
		filename= packagePath + filename;

		simAddStatusbarMessage(filename.c_str());


		 

		 
	////->creation of the mesh


	std::bitset<3> options;
	options.set(0,0);  //bit0 set: keep identical vertices
	options.set(1,0);  //bit1 set: keep identical triangles
	options.set(2,1);  //bit2 set: don't correct triangle windings

	
   std::string extension = filename.substr(filename.size()-3,filename.size());
		
	int nExtension;
	if(extension == "obj" || extension =="OBJ"){ nExtension = 0;}
	else if(extension == "dxf" || extension == "DXF"){ nExtension = 1;}
	else if(extension == "3ds" || extension == "3DS"){ nExtension = 2;}
	else if(extension == "stl" || extension == "STL"){ nExtension = 4;}
	else if(extension == "dae" || extension == "DAE"){ nExtension = 5;}
	else { return-1;}

	simInt shapeHandle = simImportShape(nExtension,filename.c_str(),options.to_ulong(),0.0001f,1.0);

// For now we disable following:
//	simSetObjectIntParameter(shapeHandle,3003,!dynamic);
//	simSetObjectIntParameter(shapeHandle,3004,collision);

	return(shapeHandle);	
}

int createPureShape(std::string size,int shapeType,bool collision,bool dynamic)
{

	//V-rep works with bounding box, and gazebo with radius
	float *sizeArray = stringToArray(size);
	if(shapeType == 1)//Sphere
	{
		sizeArray[0]=sizeArray[0]*2;
		sizeArray[1]=sizeArray[1]*2;
		sizeArray[2]=sizeArray[2]*2;
	}
    if(shapeType == 2)//Cylinder
	{
		float *sizeArray = stringToArray(size);
		sizeArray[0]=sizeArray[0]*2;
		sizeArray[1]=sizeArray[1]*2;
	}

	int options=2+4;
	if (collision)
		options+=8;
	if (!dynamic)
		options+=16;
	// for now we force a non-dynamic behaviour:
	options=22;

	simInt shapeHandle = simCreatePureShape(shapeType,options,sizeArray,0,NULL);
	return(shapeHandle);
}

float* stringToArray(std::string xyz)
{
	
	static float position[3];
	std::string buf; 
    std::stringstream ss(xyz); 

   	int i = 0;
	while (ss >> buf)
		position[i++]=getFloat(buf);
	
	return position;
}

float getFloat(const std::string& text)
{
	try
	{
		return(boost::lexical_cast<float>(text));
	}
	catch (boost::bad_lexical_cast &)
	{
		return(0.0f);
	}
}

void setObjectName(int handle,const char* name)
{
	simSetObjectName(handle,name); // can fail, but for now we don't care
	objectHandles.push_back(handle);
	char* nm=simGetObjectName(handle);
	objectNames.push_back(nm);
	simReleaseBuffer(nm);
}

int getHandleFromName(const char* name)
{
	for (int i=0;i<int(objectNames.size());i++)
	{
		if (objectNames[i].compare(name)==0)
			return(objectHandles[i]);
	}
	return(-1);
}
